@@ -11,6 +11,8 @@ import random
 import md5
 import errno
 
+from logging import debug, info, warning, error
+
 try:
 	import xml.etree.ElementTree as ET
 except ImportError:
@@ -35,13 +37,18 @@ def fixupXPath(xmlns, xpath, max = 0):
 	return retval
 
 def parseNodes(nodes, xmlns = ""):
+	## WARNING: Ignores text nodes from mixed xml/text.
+	## For instance <tag1>some text<tag2>other text</tag2></tag1>
+	## will be ignore "some text" node
 	retval = []
 	for node in nodes:
 		retval_item = {}
 		for child in node.getchildren():
 			name = stripTagXmlns(xmlns, child.tag)
-			retval_item[name] = node.findtext(".//%s" % child.tag)
-
+			if child.getchildren():
+				retval_item[name] = parseNodes([child], xmlns)
+			else:
+				retval_item[name] = node.findtext(".//%s" % child.tag)
 		retval.append(retval_item)
 	return retval
 
@@ -74,8 +81,8 @@ def dateS3toUnix(date):
 	## treats it as "localtime". Anyway...
 	return time.mktime(dateS3toPython(date))
 
-def formatSize(size, human_readable = False):
-	size = int(size)
+def formatSize(size, human_readable = False, floating_point = False):
+	size = floating_point and float(size) or int(size)
 	if human_readable:
 		coeffs = ['k', 'M', 'G', 'T']
 		coeff = ""
@@ -131,7 +138,33 @@ def mktmpfile(prefix = "/tmp/tmpfile-", randchars = 20):
 
 def hash_file_md5(filename):
 	h = md5.new()
-	f = open(filename, "r")
+	f = open(filename, "rb")
 	h.update(f.read())
 	f.close()
 	return h.hexdigest()
+
+def mkdir_with_parents(dir_name):
+	"""
+	mkdir_with_parents(dst_dir)
+	
+	Create directory 'dir_name' with all parent directories
+
+	Returns True on success, False otherwise.
+	"""
+	pathmembers = dir_name.split(os.sep)
+	tmp_stack = []
+	while pathmembers and not os.path.isdir(os.sep.join(pathmembers)):
+		tmp_stack.append(pathmembers.pop())
+	while tmp_stack:
+		pathmembers.append(tmp_stack.pop())
+		cur_dir = os.sep.join(pathmembers)
+		try:
+			debug("mkdir(%s)" % cur_dir)
+			os.mkdir(cur_dir)
+		except (OSError, IOError), e:
+			warning("%s: can not make directory: %s" % (cur_dir, e.strerror))
+			return False
+		except Exception, e:
+			warning("%s: %s" % (cur_dir, e))
+			return False
+	return True
